@@ -22,7 +22,7 @@ def rel_to_abs(x):
 
 def relative_logits_1d(q, rel_k):
     b, heads, h, w, dim = q.shape
-    logits = einsum('b h x y d, r d -> b h x y r', q, rel_k) * dim ** -0.5
+    logits = einsum('b h x y d, r d -> b h x y r', q, rel_k)
     logits = rearrange(logits, 'b h x y r -> b (h x) y r')
     logits = rel_to_abs(logits)
     logits = logits.reshape(b, heads, h, w, w)
@@ -39,14 +39,13 @@ class AbsPosEmb(nn.Module):
     ):
         super().__init__()
         scale = dim_head ** -0.5
-        self.scale = scale
         self.height = nn.Parameter(torch.randn(fmap_size, dim_head) * scale)
         self.width = nn.Parameter(torch.randn(fmap_size, dim_head) * scale)
 
     def forward(self, q):
         emb = rearrange(self.height, 'h d -> h () d') + rearrange(self.width, 'w d -> () w d')
         emb = rearrange(emb, ' h w d -> (h w) d')
-        logits = einsum('b h i d, j d -> b h i j', q, emb) * self.scale
+        logits = einsum('b h i d, j d -> b h i j', q, emb)
         return logits
 
 class RelPosEmb(nn.Module):
@@ -58,7 +57,6 @@ class RelPosEmb(nn.Module):
         super().__init__()
         scale = dim_head ** -0.5
         self.fmap_size = fmap_size
-        self.scale = scale
         self.rel_height = nn.Parameter(torch.randn(fmap_size * 2 - 1, dim_head) * scale)
         self.rel_width = nn.Parameter(torch.randn(fmap_size * 2 - 1, dim_head) * scale)
 
@@ -100,7 +98,9 @@ class Attention(nn.Module):
         q, k, v = self.to_qkv(fmap).chunk(3, dim = 1)
         q, k, v = map(lambda t: rearrange(t, 'b (h d) x y -> b h (x y) d', h = heads), (q, k, v))
 
-        sim = einsum('b h i d, b h j d -> b h i j', q, k) * self.scale
+        q *= self.scale
+
+        sim = einsum('b h i d, b h j d -> b h i j', q, k)
         sim += self.pos_emb(q)
 
         attn = sim.softmax(dim = -1)
@@ -140,23 +140,24 @@ class BottleBlock(nn.Module):
 
         # contraction and expansion
 
-        attention_dim = dim_out // proj_factor
+        attn_dim_in = dim_out // proj_factor
+        attn_dim_out = heads * dim_head
 
         self.net = nn.Sequential(
-            nn.Conv2d(dim, attention_dim, 1, bias = False),
-            nn.BatchNorm2d(attention_dim),
+            nn.Conv2d(dim, attn_dim_in, 1, bias = False),
+            nn.BatchNorm2d(attn_dim_in),
             activation,
             Attention(
-                dim = attention_dim,
+                dim = attn_dim_in,
                 fmap_size = fmap_size,
                 heads = heads,
                 dim_head = dim_head,
                 rel_pos_emb = rel_pos_emb
             ),
             nn.AvgPool2d((2, 2)) if downsample else nn.Identity(),
-            nn.BatchNorm2d(attention_dim),
+            nn.BatchNorm2d(attn_dim_out),
             activation,
-            nn.Conv2d(attention_dim, dim_out, 1, bias = False),
+            nn.Conv2d(attn_dim_out, dim_out, 1, bias = False),
             nn.BatchNorm2d(dim_out)
         )
 
@@ -220,5 +221,5 @@ class BottleStack(nn.Module):
     def forward(self, x):
         _, c, h, w = x.shape
         assert c == self.dim, f'channels of feature map {c} must match channels given at init {self.dim}'
-        assert h == self.fmap_size and w == self.fmap_size, f'height and width of feature map must match the fmap_size given at init {self.fmap_size}'
+        assert h == self.fmap_size and w == self.fmap_size, f'height and width ({h} {w}) of feature map must match the fmap_size given at init {self.fmap_size}'
         return self.net(x)
